#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import math

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release - based by codes Tin Tran Fill_rotate.py https://gimplearn.net/viewtopic.php?f=3&t=1393
#                          and Ofnuts ofn-symmetries.py (2016-09-29) https://sourceforge.net/projects/gimp-tools/files/scripts/ofn-symmetries.zip/download 


###################################################################################################################
# START fill_rotate.py Created by Tin Tran http://gimplearn.net/

def python_tt_fill_rotate_diag(new_image, layer, angle, mirror):
	pdb.gimp_image_undo_group_start(new_image)
	pdb.gimp_context_push()
	pdb.gimp_selection_none(new_image)
	#some calculation and save dimensions
	old_width,old_height = layer.width,layer.height
	new_width,new_height = layer.width*3,layer.height*3
	layer_position = pdb.gimp_image_get_item_position(new_image,layer)
	#set canvas to be 3 times in height and width of layer.
	pdb.gimp_image_resize(new_image,new_width,new_height,old_width,old_height)

	layer_copy_L = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_L,None,layer_position)
	if (mirror == 1):
		item = pdb.gimp_item_transform_flip_simple(layer_copy_L,ORIENTATION_HORIZONTAL,True,0)
		if pdb.gimp_layer_is_floating_sel(item):
			pdb.gimp_floating_sel_anchor(item)
	pdb.gimp_layer_translate(layer_copy_L,-old_width,0)
	
	layer_copy_R = pdb.gimp_layer_new_from_drawable(layer_copy_L,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_R,None,layer_position)
	pdb.gimp_layer_translate(layer_copy_R,old_width*2,0)

	layer_copy_T = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_T,None,layer_position)
	if (mirror == 1):
		item = pdb.gimp_item_transform_flip_simple(layer_copy_T,ORIENTATION_VERTICAL,True,0)
		if pdb.gimp_layer_is_floating_sel(item):
			pdb.gimp_floating_sel_anchor(item)
	pdb.gimp_layer_translate(layer_copy_T,0,-old_height)

	layer_copy_B = pdb.gimp_layer_new_from_drawable(layer_copy_T,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_B,None,layer_position)
	pdb.gimp_layer_translate(layer_copy_B,0,old_height*2)

	layer_copy_TL = pdb.gimp_layer_new_from_drawable(layer,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_TL,None,layer_position)
	if (mirror == 1):
		item = pdb.gimp_item_transform_flip_simple(layer_copy_TL,ORIENTATION_VERTICAL,True,0)
		if pdb.gimp_layer_is_floating_sel(item):
			pdb.gimp_floating_sel_anchor(item)
		item = pdb.gimp_item_transform_flip_simple(layer_copy_TL,ORIENTATION_HORIZONTAL,True,0)
		if pdb.gimp_layer_is_floating_sel(item):
			pdb.gimp_floating_sel_anchor(item)
	pdb.gimp_layer_translate(layer_copy_TL,-old_width,-old_height)

	layer_copy_TR = pdb.gimp_layer_new_from_drawable(layer_copy_TL,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_TR,None,layer_position)
	pdb.gimp_layer_translate(layer_copy_TR,old_width*2,0)

	layer_copy_BL = pdb.gimp_layer_new_from_drawable(layer_copy_TL,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_BL,None,layer_position)
	pdb.gimp_layer_translate(layer_copy_BL,0,old_height*2)

	layer_copy_BR = pdb.gimp_layer_new_from_drawable(layer_copy_TL,new_image)
	pdb.gimp_image_insert_layer(new_image,layer_copy_BR,None,layer_position)
	pdb.gimp_layer_translate(layer_copy_BR,old_width*2,old_height*2)

	#merge down them all to one original layer.
	BR_merged = pdb.gimp_image_merge_down(new_image,layer_copy_BR,EXPAND_AS_NECESSARY)
	BL_merged = pdb.gimp_image_merge_down(new_image,BR_merged,EXPAND_AS_NECESSARY)
	TR_merged = pdb.gimp_image_merge_down(new_image,BL_merged,EXPAND_AS_NECESSARY)
	TL_merged = pdb.gimp_image_merge_down(new_image,TR_merged,EXPAND_AS_NECESSARY)
	B_merged = pdb.gimp_image_merge_down(new_image,TL_merged,EXPAND_AS_NECESSARY)
	T_merged = pdb.gimp_image_merge_down(new_image,B_merged,EXPAND_AS_NECESSARY)
	R_merged = pdb.gimp_image_merge_down(new_image,T_merged,EXPAND_AS_NECESSARY)
	L_merged = pdb.gimp_image_merge_down(new_image,R_merged,EXPAND_AS_NECESSARY)
	ALL_merged = L_merged

	angle = angle/180.0 * math.pi
	item = pdb.gimp_item_transform_rotate(ALL_merged,angle,True,0,0)
	if pdb.gimp_layer_is_floating_sel(item):
		pdb.gimp_floating_sel_anchor(item)

	#set to original dimensions
	pdb.gimp_image_resize(new_image,old_width,old_height,-old_width,-old_height)
	pdb.gimp_layer_resize_to_image_size(ALL_merged)
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(new_image)
	pdb.gimp_displays_flush()
	
# END fill_rotate.py
###################################################################################################################

def plugin_diagon(img, drawable, prep, sel, rot):
	
	W = pdb.gimp_image_width(img)   #szer
	H = pdb.gimp_image_height(img)  #wys
	selection = pdb.gimp_selection_bounds(img)
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];

	pdb.gimp_selection_none(img)
	pdb.gimp_edit_copy(drawable)
	copy = pdb.gimp_edit_paste(drawable ,1)
	pdb.gimp_floating_sel_to_layer(copy)
	pdb.gimp_item_set_visible(copy, 1)
	
	wl = pdb.gimp_drawable_width(copy)
	hl = pdb.gimp_drawable_height(copy)

	if wl%2==1:
		wx = wl -1
	else:
		wx = wl
	if hl%2==1:
		hx = hl -1
	else:
		hx = hl
		
	new_image = pdb.gimp_image_new(wx, hx, 0)
	display = pdb.gimp_display_new(new_image)
	active_layer = pdb.gimp_layer_new_from_drawable(copy, new_image)
	pdb.gimp_image_insert_layer(new_image, active_layer, None, 0)
	
	if prep==1:
		p= min(wx,hx)
		pdb.gimp_image_resize(new_image, p, p, 0, 0)
		pdb.gimp_layer_resize_to_image_size(active_layer)
		W=p
		H=p
		
	if prep==2:
		p= max(wx,hx)
		old_width,old_height = wx, hx
		if wx>hx:
			new_width,new_height = wx, hx*3
			layer_position = pdb.gimp_image_get_item_position(new_image,active_layer)
			pdb.gimp_image_resize(new_image,new_width,new_height,0,old_height)
			layer_copy_T = pdb.gimp_layer_new_from_drawable(active_layer,new_image)
			pdb.gimp_image_insert_layer(new_image,layer_copy_T,None,layer_position)
			pdb.gimp_item_transform_flip_simple(layer_copy_T,ORIENTATION_VERTICAL,True,0)
			pdb.gimp_layer_translate(layer_copy_T,0,-old_height)
			layer_copy_B = pdb.gimp_layer_new_from_drawable(layer_copy_T,new_image)
			pdb.gimp_image_insert_layer(new_image,layer_copy_B,None,layer_position)
			pdb.gimp_layer_translate(layer_copy_B,0,old_height*2)
			lct = pdb.gimp_image_merge_down(new_image,layer_copy_B,EXPAND_AS_NECESSARY)
			active_layer=pdb.gimp_image_merge_down(new_image,lct,EXPAND_AS_NECESSARY)
			pdb.gimp_image_resize(new_image,old_width,old_width,0,(-new_height+old_width)/2)
			pdb.gimp_layer_resize_to_image_size(active_layer)
			
		if wx<hx:
			new_width,new_height = wx*3, hx
			layer_position = pdb.gimp_image_get_item_position(new_image,active_layer)
			pdb.gimp_image_resize(new_image,new_width,new_height,old_width,0)
			layer_copy_L = pdb.gimp_layer_new_from_drawable(drawable,new_image)
			pdb.gimp_image_insert_layer(new_image,layer_copy_L,None,layer_position)
			pdb.gimp_item_transform_flip_simple(layer_copy_L,ORIENTATION_HORIZONTAL,True,0)
			layer_copy_R = pdb.gimp_layer_new_from_drawable(layer_copy_L,new_image)
			pdb.gimp_image_insert_layer(new_image,layer_copy_R,None,layer_position)
			pdb.gimp_layer_translate(layer_copy_R,old_width*2,0)
			lct = pdb.gimp_image_merge_down(new_image,layer_copy_R,EXPAND_AS_NECESSARY)
			active_layer=pdb.gimp_image_merge_down(new_image,lct,EXPAND_AS_NECESSARY)
			pdb.gimp_image_resize(new_image,old_height,old_height,(-new_width+old_height)/2, 0)
			active = pdb.gimp_layer_resize_to_image_size(active_layer)
	
		W=p
		H=p

	reg_layer = pdb.gimp_layer_new_from_drawable(active_layer, new_image)
	pdb.gimp_image_insert_layer(new_image, reg_layer, None, 0)
	
	if sel == 0:	# "Left Top"
		
		pdb.gimp_item_transform_flip_simple (reg_layer, 0, True, 1)
		pdb.gimp_item_transform_rotate_simple(reg_layer, ROTATE_90,True,0,0)
		pdb.gimp_image_select_polygon(new_image, 0, 8,(0,0, W,0, 0,H, 0,0))		
		pdb.gimp_edit_clear (reg_layer)		
		pdb.gimp_image_merge_down(new_image, reg_layer, 0)# merge_type  EXPAND-AS-NECESSARY (0), CLIP-TO-IMAGE (1), CLIP-TO-BOTTOM-LAYER (2)

	if sel == 1:  # "Right Bottom"
		
		pdb.gimp_item_transform_flip_simple (reg_layer, 0, True, 1)
		pdb.gimp_item_transform_rotate_simple(reg_layer, ROTATE_90,True,0,0)	
		pdb.gimp_image_select_polygon(new_image, 0, 8,(W,0, W,H, 0,H, W,0))
		pdb.gimp_edit_clear (reg_layer)		
		pdb.gimp_image_merge_down(new_image, reg_layer, 0)# merge_type  EXPAND-AS-NECESSARY (0), CLIP-TO-IMAGE (1), CLIP-TO-BOTTOM-LAYER (2)

	if sel == 2:	# "Left Bottom"
		
		pdb.gimp_item_transform_flip_simple (reg_layer, 0, True, 0)
		pdb.gimp_item_transform_rotate_simple(reg_layer, ROTATE_270,True,0,0)
		pdb.gimp_image_select_polygon(new_image, 0, 8,(0,0, 0,H, W,H, 0,0))
		pdb.gimp_edit_clear (reg_layer)		
		pdb.gimp_image_merge_down(new_image, reg_layer, 0)# merge_type  EXPAND-AS-NECESSARY (0), CLIP-TO-IMAGE (1), CLIP-TO-BOTTOM-LAYER (2)

	if sel == 3: # "Right Top"
	
		pdb.gimp_item_transform_flip_simple (reg_layer, 0, True, 0)
		pdb.gimp_item_transform_rotate_simple(reg_layer, ROTATE_270,True,0,0)
		pdb.gimp_image_select_polygon(new_image, 0, 8,(0,0, W,0, W,H, 0,0))
		pdb.gimp_edit_clear (reg_layer)		
		pdb.gimp_image_merge_down(new_image, reg_layer, 0)# merge_type  EXPAND-AS-NECESSARY (0), CLIP-TO-IMAGE (1), CLIP-TO-BOTTOM-LAYER (2)

	pdb.gimp_image_remove_layer(img, copy)
	pdb.gimp_selection_none(new_image)
	pdb.gimp_selection_none(img)
	x = pdb.gimp_image_get_active_layer(new_image)
	
	if rot==1:
		python_tt_fill_rotate_diag(new_image, x, 45, 1)
		
	if rot==2:
		python_tt_fill_rotate_diag(new_image, x, -45, 1)
		
	pdb.gimp_displays_flush()

register(
        "plugin_diagon",
        "Symmetry of the SQUARE (only) according to \n diagonals & fill mirror edge. Creates new image. ",
        "Symmetry of the square according to diagonals with the choice of area",
        "MrQ",
        "GPL",
        "2019",
        "<Image>/Filters/Distorts/Symmetry/Symmetry by diagonal...",
        "RGB*, GRAY*",
        [
		#(PF_IMAGE, "image", "Input image", None),
		#(PF_DRAWABLE, "drawable", "Input layer", None),\u231C   \u21B7\u21B6  \u27F2\u27F3
		(PF_OPTION, "prep", "Prepare square:", 2, [u'\u234C'" DON'T USE \n (the square exist)", u'\u2751'" CROP TO THE SHORT SIDE \n side = min (width,height)", u'\u3013'" FILL WITH REFLECTION \n side = max (width,height)"]),		
		(PF_OPTION, "sel", "Select choice of area:", 0, [u'\u25e4'" Left Top", u'\u25e2'" Right Bottom", u'\u25e3'" Left Bottom", u'\u25e5'" Right Top"," DON'T USE "]),
		(PF_OPTION, "rot", "Rotate (°) & mirror edges:", 2, [" DON'T USE", u'\u21BB'"   45"u'\u00B0',u'\u21BA'" - 45"u'\u00B0'])         
		],
        [],
        plugin_diagon)

main()